
module spi_slave(input logic sck, // from master 
	input logic sdo, // from master
	output logic sdi, // to master
	input logic reset,
	input logic [7:0] d, // data to send 
	output logic [7:0] q); // data received

logic [2:0] cnt; 
logic qdelayed;

// 3-bit counter tracks when full byte is transmitted and new d should be sent
always_ff @(negedge sck, posedge reset) 
	if (reset) cnt = 0;
	else cnt = cnt + 3'b1;

// loadable shift register
// loads d at the start, shifts sdo into bottom position on subsequent step 
always_ff @(posedge sck)
	q <= (cnt == 0) ? d : {q[6:0], sdo};

// align sdi to falling edge of sck // load d at the start
always_ff @(negedge sck)
	qdelayed = q[7];

assign sdi = (cnt == 0) ? d[7] : qdelayed;

endmodule